# Written by Alexandria D'Souza and Birgit Schilling
#
# All publications that utilize this software DIAProbe
# should provide acknowledgement to the Buck Institute for Research on Aging,
# the Gibson Laboratory/Chemistry Core

import re
import numpy
import numpy as np
from scipy import stats
import scipy.stats as st
import sys
import csv
import os
import subprocess
import shutil

try:
    tempRfolder = sys.argv[1].strip("DIAprobe_MS1Probe_Input.csv")
    if not os.path.exists(tempRfolder + "tempR"):
        os.makedirs(tempRfolder + "tempR")
    fn = sys.argv[1]
    
    
except:
    raw_input('Please make sure you are using the correct External Tool from Skyline. Press Enter.')
    sys.exit()

with open(fn, 'rb') as myfile:
    skylineFile = csv.reader(myfile, dialect='excel')
    data = []
    for row in skylineFile:
        data.append(row)
    myfile.close()

try:
    #reading in the necessary information into a 1 by 1 array
    peakAreaCol = data[0].index("light Area")
    peakAreas2 = [row[peakAreaCol] for row in data[1 :]]
    x=0
    peakAreas=[]
    while x<len(peakAreas2):
        if peakAreas2[x] == '#N/A':
            peakAreas.append('N/A')
    
        else:
            peakAreas.append(float(peakAreas2[x]))
        x+=1

    ionNamesCol = data[0].index("FragmentIon")
    ionNames = [row[ionNamesCol] for row in data[1 :]]

    peptideNamesCol = data[0].index("PeptideSequence")
    peptideNames = [row[peptideNamesCol] for row in data[1 :]]

    lightprecMZCol = data[0].index("light PrecursorMz")
    lprecs = [row[lightprecMZCol] for row in data[1 :]]

    lightprodMZCol = data[0].index("light ProductMz")
    lprods = [row[lightprodMZCol] for row in data[1 :]]

    isoCol = data[0].index("IsotopeDistRank")
    iso = [row[isoCol] for row in data[1 :]]

    rname = data[0].index("ReplicateName")
    repname = [row[rname] for row in data[1 :]]

    modseq = data[0].index("light ModifiedSequence")
    mseq = [row[modseq] for row in data[1 :]]

    prch = data[0].index("PrecursorCharge")
    precch = [row[prch] for row in data[1 :]]

    proch = data[0].index("ProductCharge")
    prodch = [row[proch] for row in data[1 :]]

    protname = data[0].index("ProteinName")
    proname = [row[protname] for row in data[1 :]]

    rettime = data[0].index("light RetentionTime")
    rtime = [row[rettime] for row in data[1 :]]

    librank = data[0].index("LibraryRank")
    lrank = [row[librank] for row in data[1 :]]

    protDesc = data[0].index("ProteinDescription")
    pdes = [row[protDesc] for row in data[1 :]]

    conditions = data[0].index("Condition")
    condData = [row[conditions] for row in data[1 :]]

    BioRep = data[0].index("BioReplicate")
    breps = [row[BioRep] for row in data[1 :]]

    print breps[6]

    
    if "good signal" in data[0]:
        goodsigCol = data[0].index("good signal")
        gsig = [row[goodsigCol] for row in data[1:]]
    else:
        gsig = []
    
    if "do not use" in data[0]:
        donotuseCol = data[0].index("do not use")
        dnu = [row[donotuseCol] for row in data[1:]]
    else:
        dnu = []
    if "light PrecursorNote" in data[0]:
        pnCol = data[0].index("light PrecursorNote")
        precnotes = [row[pnCol] for row in data[1 :]]
    else:
        precnotes=[]
except:
    raw_input('Please use the MS1Filtering_09_2012 input report, press any key then press enter to exit.')
    sys.exit()

# Copyright (C) 2008 Simone Leo - CRS4. All Rights Reserved.
# 
# Permission to use, copy, modify, and distribute this software and its
# documentation for educational, research, and not-for-profit purposes, without
# fee and without a signed licensing agreement, is hereby granted, provided
# that the above copyright notice, this paragraph and the following two
# paragraphs appear in all copies, modifications, and distributions. Contact
# CRS4, Parco Scientifico e Tecnologico, Edificio 1, 09010 PULA (CA - Italy),
# +3907092501 for commercial licensing opportunities.
# 
# IN NO EVENT SHALL CRS4 BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
# INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF
# THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF CRS4 HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# CRS4 SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED
# HEREUNDER IS PROVIDED "AS IS". CRS4 HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

"""
Functions for computing the False Discovery Rate (FDR) of a multiple test
procedure. The FDR is the expected proportion of falsely rejected hypotheses.

@see:
  Yoav Benjamini and Yosef Hochberg, I{Controlling the False Discovery Rate:
  A Practical and Powerful Approach to Multiple Testing}. Journal of the Royal
  Statistical Society, Series B (Methodological), Vol. 57, No. 1 (1995), pp.
  289-300.

  Yekutieli, D. and Benjamini, Y., I{Resampling-based false discovery rate
  controlling multiple test procedures for correlated test statistics}. J. of
  Statistical Planning and Inference, 82, pp. 171-196, 1999.

  John D. Storey (2002), I{A direct approach to false discovery rates}. Journal
  of the Royal Statistical Society: Series B (Statistical Methodology) 64 (3),
  479-498.
"""

__docformat__ = 'epytext en'


import operator
import random


def bh_rejected(pv, threshold):
    """
    Return the list of rejected p-values from C{pv} at FDR level C{threshold}.

    The downside of this approach is that the FDR level must be chosen in
    advance - the L{bh_qvalues} function yields a p-value-like output instead.

    @type pv: list
    @param pv: p-values from a multiple statistical test
    @type threshold: float
    @param threshold: the level at which FDR rate should be controlled

    @rtype: list
    @return: p-values of rejected null hypotheses
    """
    if threshold < 0 or threshold > 1:
        raise ValueError("the threshold must be between 0 and 1")
    if not pv:
        return []
    pv = sorted(pv)
    if pv[0] < 0 or pv[-1] > 1:
        raise ValueError("p-values must be between 0 and 1")
    m = len(pv)
    for i in xrange(m-1, -1, -1):
        if pv[i] <= float(i+1)*threshold/m:
            return pv[:i+1]
    return []


def bh_qvalues(pv):
    """
    Return Benjamini-Hochberg FDR q-values corresponding to p-values C{pv}.

    This function implements an algorithm equivalent to L{bh_rejected} but
    yields a list of 'adjusted p-values', allowing for rejection decisions
    based on any given threshold.

    @type pv: list
    @param pv: p-values from a multiple statistical test

    @rtype: list
    @return: adjusted p-values to be compared directly with the desired FDR
      level
    """
    if not pv:
        return []
    m = len(pv)
    args, pv = zip(*sorted(enumerate(pv), None, operator.itemgetter(1)))
    if pv[0] < 0 or pv[-1] > 1:
        raise ValueError("p-values must be between 0 and 1")
    qvalues = m * [0]
    mincoeff = pv[-1]
    qvalues[args[-1]] = mincoeff
    for j in xrange(m-2, -1, -1):
        coeff = m*pv[j]/float(j+1)
        if coeff < mincoeff:
            mincoeff = coeff
        qvalues[args[j]] = mincoeff
    return qvalues


def get_pi0(pv, lambdas):
    """
    Compute Storey's C{pi0} from p-values C{pv} and C{lambda}.

    this function is equivalent to::
    
        m = len(pv)
        return [sum(p >= l for p in pv)/((1.0-l) * m) for l in lambdas]
        
    but the above is C{O(m*n)}, while it needs only be C{O(m+n)
    (n = len(lambdas))}

    @type pv: list
    @param pv: B{SORTED} p-values vector
    @type lambdas: list
    @param lambdas: B{SORTED} lambda values vector

    @rtype: list
    @return: estimated proportion of null hypotheses C{pi0} for each lambda
    """
    m = len(pv)
    i = m - 1
    pi0 = []
    for l in reversed(lambdas):
        while i >= 0 and pv[i] >= l:
            i -= 1
        pi0.append((m-i-1)/((1.0-l)*m))
    pi0.reverse()
    return pi0


def storey_qvalues(pv, l=None):
    """
    Return Storey FDR q-values corresponding to p-values C{pv}.

    The main difference between B-H's and Storey's q-values is that the latter
    are weighted by the estimated proportion C{pi0} of true null hypotheses.

    @type pv: list
    @param pv: p-values from a multiple statistical test
    @type l: float
    @param l: lambda value for C{pi0} (fraction of null p-values) estimation

    @rtype: list
    @return: storey q-values corresponding to C{pv}
    """
    if not pv:
        return []
    m = len(pv)
    args, pv = zip(*sorted(enumerate(pv), None, operator.itemgetter(1)))
    if pv[0] < 0 or pv[-1] > 1:
        raise ValueError("p-values must be between 0 and 1")

    if l is None:
        # optimal lambda/pi0 estimation
        lambdas = [i/100.0 for i in xrange(0, 91, 5)]
        n = len(lambdas)
        pi0 = get_pi0(pv, lambdas)
        min_pi0 = min(pi0)
        mse = [0] * n        
        for i in xrange(1, 101):
            # compute bootstrap sample with replacement
            pv_boot = [pv[int(random.random()*m)] for j in xrange(m)]
            pi0_boot = get_pi0(sorted(pv_boot), lambdas)
            for j in xrange(n):
                mse[j] += (pi0_boot[j] - min_pi0) * (pi0_boot[j] - min_pi0)
        min_mse = min(mse)
        argmin_mse = [i for i, mse_i in enumerate(mse) if mse_i == min_mse]
        pi0 = min(pi0[i] for i in argmin_mse)
        pi0 = min(pi0, 1)
    else:
        try:
            l = float(l)
        except ValueError:
            raise TypeError("lambda must be a number")
        if l < 0 or l >= 1:
            raise ValueError("lambda must be within [0,1)")
        pi0 = get_pi0(pv, [l])
        pi0 = min(pi0[0], 1)

    qvalues = m * [0]
    mincoeff = pi0 * pv[-1]
    qvalues[args[-1]] = mincoeff
    for j in xrange(m-2, -1, -1):
        coeff = pi0*m*pv[j]/float(j+1)
        if coeff < mincoeff:
            mincoeff = coeff
        qvalues[args[j]] = mincoeff
    return qvalues


KO=[]
WT=[]

ratioAr=[]
pvalueAr=[]
gsigAr=[[]]
dnuAr=[[]]
yList=[]
indList=[]
realList=[]
i=0
j=0
y=[]
posList=[]
indexList=[]
j=0
k=1
precAr=[[]]
repNameAr=[[]]
pepAr=[[]]
sucAr=[[]]
ionAr=[[]]
noteAr=[[]]
rankAr=[[]]
mseqAr=[[]]
precMzAr=[[]]
prodMzAr=[[]]
proNameAr=[[]]
precChAr=[[]]
precNoteAr=[[]]
prodChAr=[[]]
rtAr=[[]]
lrankAr=[[]]
prodesAr=[[]]
precNoteAr=[[]]
conDataAr=[[]]
brepsAr=[[]]
z=0
raAr=[]
condAr=[]

conditions = 2
x=0
condAr.append(sys.argv[5])
condAr.append(sys.argv[6])

raAr.append(sys.argv[8])
raAr.append(sys.argv[9])



numRatio = 1
try:
    #splitting up the peakareas and corresponding properties from skyline
    #into a list of lists based on the product m/z since this is what varies
    #between the transitions.
    while k<len(lprods):   
        if all([lprods[j] == lprods[k], mseq[j] == mseq[k], precch[j] == precch[k]]):
            precAr[z].append(peakAreas[j])
            repNameAr[z].append(repname[j])
            pepAr[z].append(peptideNames[j])
            ionAr[z].append(ionNames[j])
            mseqAr[z].append(mseq[j])
            rankAr[z].append(iso[j])
            precMzAr[z].append(lprecs[j])
            prodMzAr[z].append(lprods[j])
            proNameAr[z].append(proname[j])
            precChAr[z].append(precch[j])
            prodChAr[z].append(prodch[j])
            rtAr[z].append(rtime[j])
            lrankAr[z].append(lrank[j])
            prodesAr[z].append(pdes[j])
            conDataAr[z].append(condData[j])
            brepsAr[z].append(breps[j])
            if gsig != []:
                gsigAr[z].append(gsig[j])
            if dnu != []:
                dnuAr[z].append(dnu[j])
            if precnotes != []:
                precNoteAr[z].append(precnotes[j])
            if k == len(lprods)-1:
                precAr[z].append(peakAreas[k])
                repNameAr[z].append(repname[k])
                pepAr[z].append(peptideNames[k])
                ionAr[z].append(ionNames[k])
                mseqAr[z].append(mseq[k])
                rankAr[z].append(iso[k])
                precMzAr[z].append(lprecs[k])
                prodMzAr[z].append(lprods[k])
                proNameAr[z].append(proname[k])
                precChAr[z].append(precch[k])
                prodChAr[z].append(prodch[k])
                rtAr[z].append(rtime[k])
                lrankAr[z].append(lrank[k])
                prodesAr[z].append(pdes[k])
                conDataAr[z].append(condData[k])
                brepsAr[z].append(breps[k])
                if gsig != []:
                    gsigAr[z].append(gsig[k])
                if dnu != []:
                    dnuAr[z].append(dnu[k])
                if precnotes != []:
                    precNoteAr[z].append(precnotes[k])
        elif any([lprods[j] != lprods[k], mseq[j] != mseq[k], precch[j] != precch[k]]):
            precAr[z].append(peakAreas[k-1])
            repNameAr[z].append(repname[k-1])
            pepAr[z].append(peptideNames[k-1])
            ionAr[z].append(ionNames[k-1])
            mseqAr[z].append(mseq[k-1])
            rankAr[z].append(iso[k-1])
            precMzAr[z].append(lprecs[k-1])
            prodMzAr[z].append(lprods[k-1])
            proNameAr[z].append(proname[k-1])
            precChAr[z].append(precch[k-1])
            prodChAr[z].append(prodch[k-1])
            rtAr[z].append(rtime[k-1])
            lrankAr[z].append(lrank[k-1])
            prodesAr[z].append(pdes[k-1])
            conDataAr[z].append(condData[k-1])
            brepsAr[z].append(breps[k-1])
            if gsig != []:
                gsigAr[z].append(gsig[k-1])
            if dnu != []:
                dnuAr[z].append(dnu[k-1])
            if precnotes != []:
                precNoteAr[z].append(precnotes[k-1])
            precAr.append([])
            repNameAr.append([])
            pepAr.append([])
            ionAr.append([])
            mseqAr.append([])
            rankAr.append([])
            precMzAr.append([])
            prodMzAr.append([])
            proNameAr.append([])
            precChAr.append([])
            prodChAr.append([])
            rtAr.append([])
            lrankAr.append([])
            prodesAr.append([])
            conDataAr.append([])
            brepsAr.append([])
            if gsig != []:
                gsigAr.append([])
            if dnu != []:
                dnuAr.append([])
            if precnotes != []:
                precNoteAr.append([])
            z+=1  
        else:
            break
        j+=1
        k+=1

    while k<len(lprecs):
        l=0
        if all([lprods[j] == lprods[k], mseq[j] == mseq[k], precch[j] == precch[k]]):
            precAr[z].append(peakAreas[j])
            conDataAr[z].append(condData[j])
            brepsAr[z].append(breps[j])
            if k == len(lprods)-1:
                precAr[z].append(peakAreas[k])
                conDataAr[z].append(condData[k])
                brepsAr[z].append(breps[k])
        elif any([lprods[j] != lprods[k], mseq[j] != mseq[k], precch[j] != precch[k]]):
            precAr[z].append(peakAreas[k-1])
            conDataAr[z].append(condData[k-1])
            brepsAr[z].append(breps[k-1])
            precAr.append([])
            conDataAr.append([])
            brepsAr.append([])
            z+=1
        elif k == len(lprods)-1:
            precAr[z].append(peakAreas[k])
            conDataAr[z].append(condData[k])
            brepsAr[z].append(breps[k])
            z+=1
        else:
            break
        j+=1
        k+=1
except:
    raw_input('Unsuccesful, check your peak areas or your condition descriptions. Please try again')
    sys.exit()

   
def t_test(array1, array2):
    num1 = len(array1)
    num2 = len(array2)
    var1 = np.var(array1, ddof=1)
    var2 = np.var(array2, ddof=1)  
    t_stat = (np.mean(array1) - np.mean(array2)) / np.sqrt(var1/num1 + var2/num2)
    df = ((var1/num1 + var2/num2)**(2.0))/((var1/num1)**(2.0)/(num1-1) + (var2/num2)**(2.0)/(num2-1)) 
    one_tailed_p_value = 1.0 - st.t.cdf(t_stat,df)
    two_tailed_p_value = 1.0 - ( st.t.cdf(np.abs(t_stat),df) - st.t.cdf(-np.abs(t_stat),df) )    
    t_ind, p_ind = st.ttest_ind(array1, array2)
    return two_tailed_p_value

i=0
WT=[]
KO=[]
KOthenWT=[]
assign=[]
k=0
#based on the description provided, the information is split up into a list
#of lists with the KO data first and WT following.

y=0
ratioIndex=[]
try:
    while y<len(raAr):
        z=0
        while z<len(condAr):
            if condAr[z] == raAr[y]:
                ratioIndex.append(z)
            z+=1
        y+=1

    assign2=[]
    i=0
    while i<len(precAr):
        assign2=[]
        k=0
        while k<len(condAr):
            j=0
            assign=[]
            while j<len((precAr[i])):            
                if condAr[k] in repNameAr[i][j]:
                    assign.append(precAr[i][j])            
                j+=1            
            assign2.append(assign)
            k+=1
        KOthenWT.append(assign2)
        i+=1
except:
    raw_input('Check the user input conditions and try again.')
    sys.exit()

w=0
KOthenWT2=[]
try:
    while w<len(KOthenWT):
        y=0
        kos=[]
        while y<len(KOthenWT[w]):
            x=0
            kos2=[]
            while x<len(KOthenWT[w][y]):        
                if all ([KOthenWT[w][y][x] != 'N/A', KOthenWT[w][y][x] != 0]):
                    kos2.append(KOthenWT[w][y][x])
                x+=1
            kos.append(kos2)
            y+=1
        KOthenWT2.append(kos)
        w+=1
except:
    raw_input('There are too many zero readings for Peak Areas.')
    sys.exit()

try:
    MnWT=[]
    SdWT=[]
    CvWT=[]
    MnKO=[]
    SdKO=[]
    CvKO=[]
    w=0
    k=0
    ratioAr=[]
    pvalueAr=[]
    #calculating the mean, sd, and cv for the data of KO and WT.
    #calculates the ratio of KO mean:WT mean and then calculates the
    #pvalues of these lists
    while w<len(KOthenWT2):
        y=0
        while y<len(KOthenWT2[w]):        
            wtmean = np.mean(KOthenWT2[w][y])
            MnWT.append(wtmean)
            sdArray=[]
            r=0
            while r <len(KOthenWT2[w][y]):
                sdArray.append(np.square((KOthenWT2[w][y][r]-np.mean(KOthenWT2[w][y]))))
                r+=1
            wtsd = np.sqrt((sum(sdArray))/(len(KOthenWT2[w][y])-1))
            np.seterr(all='ignore') 
            SdWT.append(wtsd)
            if wtmean != 0:
                wtcv = (wtsd/wtmean)*100
            else:
                wtcv =0
            CvWT.append(wtcv)
            y+=1
        if np.mean(KOthenWT2[w][1]) != 0:
            a=  np.mean(KOthenWT2[w][0])
            b=  np.mean(KOthenWT2[w][1])
            c=a/b
            ratioAr.append(c)
        else:
            ratioAr.append(0)
        try:
            pv=t_test(KOthenWT2[w][0], KOthenWT2[w][1])
            pvalueAr.append(pv)
        except:
            pvalueAr.append(0)
        qvalueList=bh_qvalues(pvalueAr)
        w+=1

    j=0
    k=1
    z=0
    sumFrag=[[]]
    sumMz=[[]]
    sumMod=[[]]
    sumCh=[[]]
    while k<len(precMzAr):   
        if all([precMzAr[j][0] == precMzAr[k][0], mseqAr[j][0] == mseqAr[k][0], precChAr[j][0] == precChAr[k][0]]):
            if not 'precursor' in ionAr[j][0]:
                sumFrag[z].append(KOthenWT[j])
                sumMz[z].append(precMzAr[j][0])
                sumMod[z].append(mseqAr[j][0])
                sumCh[z].append(precChAr[j][0])
                if k == len(precMzAr)-1:
                    if not 'precursor' in ionAr[k][0]:
                        sumFrag[z].append(KOthenWT[k])
                        sumMz[z].append(precMzAr[k][0])
                        sumMod[z].append(mseqAr[k][0])
                        sumCh[z].append(precChAr[k][0])
        elif any([precMzAr[j][0] != precMzAr[k][0], mseqAr[j][0] != mseqAr[k][0], precChAr[j][0] != precChAr[k][0]]):
            #if all([ionAr[k-1][0] != 'precursor', ionAr[k-1][0] != 'precursor [M+1]', ionAr[j][0] != 'precursor [M+2]']):
            if not 'precursor' in ionAr[k-1][0]:
                sumFrag[z].append(KOthenWT[k-1])
                sumMz[z].append(precMzAr[k-1][0])
                sumMod[z].append(mseqAr[k-1][0])
                sumCh[z].append(precChAr[k-1][0])
                sumFrag.append([])
                sumMz.append([])
                sumMod.append([])
                sumCh.append([])
                z+=1
        elif k == len(precMzAr)-1:
            #if all([ionAr[j][0] != 'precursor', ionAr[j][0] != 'precursor [M+1]', ionAr[j][0] != 'precursor [M+2]']):
            if not 'precursor' in ionAr[k][0]:
                sumFrag[z].append(KOthenWT[k])
                sumMz[z].append(precMzAr[k][0])
                sumMod[z].append(mseqAr[k][0])
                sumCh[z].append(precChAr[k][0])
                sumFrag.append([])
                sumMz.append([])
                sumMod.append([])
                sumCh.append([])
                z+=1
        else:
            break
        j+=1
        k+=1

    i=0
    sums=[]
    sums2=[]
    while i<len(sumFrag):
        j=0
        buffer3=[]
        buffer4=[]
        while j<len(sumFrag[i]):    
            k=0
            buffer1=[]
            buffer2=[]
            while k==0:
                buffer1 += sumFrag[i][j][k]
                k+=1
            while k == 1:
                buffer2 += sumFrag[i][j][k]
                k+=1
            buffer3.append(buffer1)
            buffer4.append(buffer2)   
            j+=1
        sums.append(buffer3)
        sums2.append(buffer4)
        i+=1

    g=0
    sumMn=[]
    sumSd=[]
    sumCv=[]
    sumMn2=[]
    sumSd2=[]
    sumCv2=[]
    sumRatio=[]
    sumPV=[]
    while g<len(sums):
        sumsBuf=[]
        sumsBuf2=[]
        sumsBef = [sum(x) for x in zip(*sums[g])]
        sumsBef2 = [sum(x) for x in zip(*sums2[g])]
        for x in sumsBef:
            sumsBuf.append(x)
        for y in sumsBef2:
            sumsBuf2.append(y)
        if len(sumsBuf) != 0:
            smfr = (sum(sumsBuf))/len(sumsBuf)
        else:
            smfr = 0
        sumMn.append(smfr)
        sdArray=[]
        r=0
        while r < len(sumsBuf):
            sdArray.append(np.square(sumsBuf[r]-np.mean(sumsBuf)))
            r+=1
        sdfr = np.sqrt((sum(sdArray))/(len(sumsBuf)-1))
        sumSd.append(sdfr)
        cvfr = (sdfr/smfr)*100
        sumCv.append(cvfr)
        if len(sumsBuf2) != 0:
            smfr2 = (sum(sumsBuf2))/len(sumsBuf2)
        else:
            smfr2 = 0
        sumMn2.append(smfr2)
        sdArray2=[]
        s=0
        while s < len(sumsBuf2):
            sdArray2.append(np.square(sumsBuf2[s]-np.mean(sumsBuf2)))
            s+=1
        sdfr2 = np.sqrt((sum(sdArray2))/(len(sumsBuf2)-1))
        sumSd2.append(sdfr2)
        cvfr2 = (sdfr2/smfr2)*100
        sumCv2.append(cvfr2)
        if smfr2 != 0:
            ratioSum = smfr/smfr2
        else:
            ratioSum='N/A'
        sumRatio.append(ratioSum)
        try:
            pvSum=t_test(sumsBuf, sumsBuf2)
        except ZeroDivisionError:
            pvSum=0
        
        sumPV.append(pvSum)
        sumQV=bh_qvalues(sumPV)
        g+=1


    

    
    labelAr=[]
    labelAr.append('Peptide')
    labelAr.append('Transition')
    labelAr.append('Rank')
    labelAr.append('LibraryRank')
    labelAr.append('Modified Sequence')
    labelAr.append('Precursor M/Z')
    labelAr.append('Product M/Z')
    labelAr.append('Protein Name')
    labelAr.append('Protein Description')
    labelAr.append('Precursor Charge')
    labelAr.append('Product Charge')
    labelAr.append('Retention Time')
    u=0
    while u<len(condAr):
        labelAr.append('Mean '+condAr[u])
        labelAr.append('Stan Dev '+condAr[u])
        labelAr.append('CV '+condAr[u])
        u+=1
    v=0
    while v+1<len(raAr):
        labelAr.append('Ratio '+raAr[v]+'/'+raAr[v+1])
        labelAr.append('P-Value '+raAr[v]+', '+raAr[v+1])
        labelAr.append('Q-Value '+raAr[v]+', '+raAr[v+1])
        v+=2

    labelAr.append('good signal')
    labelAr.append('do not use')
    labelAr.append('Precursor Note')
    ry=0
    mnsAr=[]
    i=0
    i=0
    j=0
    a=0
    b=1
    c=2
    u=0
    while ry<len(pepAr):
        thisAr1=[]
        thisAr1.append(pepAr[ry][1])
        thisAr1.append(ionAr[ry][1])
        thisAr1.append(rankAr[ry][1])
        thisAr1.append(lrankAr[ry][1])
        thisAr1.append(mseqAr[ry][1])
        thisAr1.append(precMzAr[ry][1])
        thisAr1.append(prodMzAr[ry][1])
        thisAr1.append(proNameAr[ry][1])
        thisAr1.append(prodesAr[ry][1])
        thisAr1.append(precChAr[ry][1])
        thisAr1.append(prodChAr[ry][1])
        thisAr1.append(rtAr[ry][1])
        x=0
        while x < conditions:
            thisAr1.append(MnWT[i])
            thisAr1.append(SdWT[i])
            thisAr1.append(CvWT[i])
            i+=1  
            x+=1
        e=0
        while e<1:
            thisAr1.append(ratioAr[j])
            thisAr1.append(pvalueAr[j])
            thisAr1.append(qvalueList[j])
            j+=1
            e+=1
        if gsig != []:
            thisAr1.append(gsigAr[ry][1])
        if dnu != []:
            thisAr1.append(dnuAr[ry][1])
        if precnotes != []:
            thisAr1.append(precNoteAr[ry][1])
        thisAr1.append('')
        ry+=1
        b+=3
        c+=3
        mnsAr.append(thisAr1)

    precursorMz=[]
    proNames=[]
    KO=[]
    WT=[]
    appendInfo = sys.argv[10]
    fn = sys.argv[3] + '\\' + sys.argv[2] + '_stat_' + appendInfo + '.csv'
    with open(fn, 'wb') as myfile:
        outputFile = csv.writer(myfile)
        i=0
        j=1
        u=0
        
        outputFile.writerow(labelAr)
        while i<len(mnsAr):

            outputFile.writerows([mnsAr[i]])
            try:
                if mnsAr[i][1] == 'precursor [M+2]':
                    if mnsAr[i+1][1] == 'precursor':
                        donothing=0
                elif all([mnsAr[i][4] == sumMod[u][0], mnsAr[i][5] == sumMz[u][0], mnsAr[i][9] == sumCh[u][0]]):
                    if any([mnsAr[i][4] != mnsAr[i+1][4], mnsAr[i][5] != mnsAr[i+1][5], mnsAr[i][9] != mnsAr[i+1][9], mnsAr[i+1][1] == 'precursor']):
                        outputFile.writerow([mnsAr[i][0],'sum of fragment ions','','',mnsAr[i][4],mnsAr[i][5],'',mnsAr[i][7],mnsAr[i][8],''\
                                          ,'','',sumMn[u],sumSd[u],sumCv[u],sumMn2[u]\
                                             ,sumSd2[u],sumCv2[u],sumRatio[u], sumPV[u], sumQV[u], mnsAr[i][21], mnsAr[i][22]])
                        precursorMz.append(mnsAr[i][5])
                        proNames.append(mnsAr[i][7])
                        KO.append(sumMn[u])
                        WT.append(sumMn2[u])
                        u+=1
            except IndexError:
                outputFile.writerow([mnsAr[i][0],'sum of fragment ions','','',mnsAr[i][4],mnsAr[i][5],'',mnsAr[i][7],mnsAr[i][8],''\
                                          ,'','',sumMn[len(sums)-1],sumSd[len(sums)-1]\
                                     ,sumCv[len(sums)-1],sumMn2[len(sums)-1],sumSd2[len(sums)-1]\
                                     ,sumCv2[len(sums)-1],sumRatio[len(sums)-1], sumPV[len(sums)-1], sumQV[len(sums)-1], mnsAr[i][21], mnsAr[i][22]])
                precursorMz.append(mnsAr[i][5])
                proNames.append(mnsAr[i][7])
                KO.append(sumMn[len(sums)-1])
                WT.append(sumMn2[len(sums)-1])
            i+=1  
        myfile.close()
    #allows the user to name the Skyline report, this is placed into the same
    #folder as the Skyline file
    appendInfo2 = sys.argv[10] #raw_input('What would you like to append to the Skyline report? ')
    fnTwo= sys.argv[3] + '\\' + sys.argv[2] + '_' + appendInfo2 + '.csv'
    fnThree = sys.argv[3] + '\\' + sys.argv[2]
    with open(fnTwo, 'wb') as inputFile:
        inputfn = csv.writer(inputFile)
        i=0
        while i<len(data):
            inputfn.writerows([data[i]])
            i+=1        
        inputFile.close()
    os.startfile(fn)
    



    newNames="".join(proNames)
    fragi = newNames.split('|')

    #this splits the protein names and takes out the actual protein name
    #without the other uniprot info

    ti=0
    pn=[]
    while ti<len(proNames):
        pnEdit=proNames[ti].split('|')
        pn.append(pnEdit[2])
        ti+=1

    #The accession number of the framents are also appended
    accAr=[]
    i=1
    while i< len(fragi):
        accAr.append(fragi[i])
        i+=2
    KOzscore=list(stats.zscore(KO))
    WTzscore=list(stats.zscore(WT))
    i=0
    dataFinal=[]  
     
    allData=[]
    while i<len(sumRatio):
        if sumQV[i] < 0.01:
            accmz = accAr[i] + ' | ' + precursorMz[i] + ' Mz'
            logRatio= np.log2(sumRatio[i])
            #dataFinal.append([accAr[i], precursorMz[i], KO[i], WT[i], sumRatio[i],sumQV[i]])
            allData.append([accAr[i], accmz, KO[i], WT[i], sumRatio[i],sumQV[i], KOzscore[i], WTzscore[i], logRatio])
            
        i+=1

    fn = tempRfolder + "tempR" + "\\" + "stats_Rscript.csv"
    with open(fn, 'wb') as myfile:
        outputFile = csv.writer(myfile)
        i=0
        
        
        outputFile.writerow(['Protein', 'Protein MZ', 'Condition 1', 'Condition 2', 'Ratio', 'Qvalue', 'Zscore C1', 'Zscore C2', 'LogRatio'])
        while i<len(allData):
            
            outputFile.writerows([allData[i]])

            
            i+=1
            
        myfile.close()
        
except:
    print 'Your Statistics File has not been successfully written. Try again.'


#####Protein Ratios
j=0
k=1
precArProt=[[]]
repNameArProt=[[]]
pepArProt=[[]]
sucArProt=[[]]
ionArProt=[[]]
noteArProt=[[]]
rankArProt=[[]]
mseqArProt=[[]]
precMzArProt=[[]]
prodMzArProt=[[]]
proNameArProt=[[]]
precChArProt=[[]]
precNoteArProt=[[]]
prodChArProt=[[]]
rtArProt=[[]]
lrankArProt=[[]]
proDesArProt=[[]]
precNoteArProt=[[]]
conDataArProt=[[]]
brepsArProt=[[]]
z=0
while k<len(lprods):
        l=0
        if proname[j] == proname[k]:
            precArProt[z].append(peakAreas[j])
            repNameArProt[z].append(repname[j])
            pepArProt[z].append(peptideNames[j])
            ionArProt[z].append(ionNames[j])
            mseqArProt[z].append(mseq[j])
            rankArProt[z].append(iso[j])
            precMzArProt[z].append(lprecs[j])
            prodMzArProt[z].append(lprods[j])
            proNameArProt[z].append(proname[j])
            precChArProt[z].append(precch[j])
            prodChArProt[z].append(prodch[j])
            rtArProt[z].append(rtime[j])
            lrankArProt[z].append(lrank[j])
            proDesArProt[z].append(pdes[j])
            conDataArProt[z].append(condData[j])
            brepsArProt[z].append(breps[j])
            if k == len(proname)-1:
                precArProt[z].append(peakAreas[k])
                repNameArProt[z].append(repname[k])
                pepArProt[z].append(peptideNames[k])
                ionArProt[z].append(ionNames[k])
                mseqArProt[z].append(mseq[k])
                rankArProt[z].append(iso[k])
                precMzArProt[z].append(lprecs[k])
                prodMzArProt[z].append(lprods[k])
                proNameArProt[z].append(proname[k])
                precChArProt[z].append(precch[k])
                prodChArProt[z].append(prodch[k])
                lrankArProt[z].append(lrank[k])
                proDesArProt[z].append(pdes[k])
                conDataArProt[z].append(condData[k])
                brepsArProt[z].append(breps[k])
        elif proname[j] != proname[k]:
            precArProt[z].append(peakAreas[k-1])
            repNameArProt[z].append(repname[k-1])
            pepArProt[z].append(peptideNames[k-1])
            ionArProt[z].append(ionNames[k-1])
            mseqArProt[z].append(mseq[k-1])
            rankArProt[z].append(iso[k-1])
            precMzArProt[z].append(lprecs[k-1])
            prodMzArProt[z].append(lprods[k-1])
            proNameArProt[z].append(proname[k-1])
            precChArProt[z].append(precch[k-1])
            prodChArProt[z].append(prodch[k-1])
            rtArProt[z].append(rtime[k-1])
            lrankArProt[z].append(lrank[k-1])
            proDesArProt[z].append(pdes[k-1])
            conDataArProt[z].append(condData[k-1])
            brepsArProt[z].append(breps[k-1])
            precArProt.append([])
            repNameArProt.append([])
            pepArProt.append([])
            ionArProt.append([])
            mseqArProt.append([])
            rankArProt.append([])
            precMzArProt.append([])
            prodMzArProt.append([])
            proNameArProt.append([])
            precChArProt.append([])
            prodChArProt.append([])
            rtArProt.append([])
            lrankArProt.append([])
            proDesArProt.append([])
            conDataArProt.append([])
            brepsArProt.append([])
            z+=1
        elif k == len(proname)-1:
            precArProt[z].append(peakAreas[k])
            repNameArProt[z].append(repname[k])
            pepArProt[z].append(peptideNames[k])
            ionArProt[z].append(ionNames[k])
            mseqArProt[z].append(mseq[k])
            rankArProt[z].append(iso[k])
            precMzArProt[z].append(lprecs[k])
            prodMzArProt[z].append(lprods[k])
            proNameArProt[z].append(proname[k])
            precChArProt[z].append(precch[k])
            prodChArProt[z].append(prodch[k])
            lrankArProt[z].append(lrank[k])
            proDesArProt[z].append(pdes[k])
            conDataArProt[z].append(condData[k])
            brepsArProt[z].append(breps[k])
            precArProt.append([])
            repNameArProt.append([])
            pepArProt.append([])
            ionArProt.append([])
            mseqArProt.append([])
            rankArProt.append([])
            precMzArProt.append([])
            prodMzArProt.append([])
            proNameArProt.append([])
            precChArProt.append([])
            prodChArProt.append([])
            rtArProt.append([])
            lrankArProt.append([])
            proDesArProt.append([])
            conDataArProt.append([])
            brepsArProt.append([])
            z+=1
        else:
            break
        j+=1
        k+=1

while k<len(proname):
    l=0
    if proname[j] == proname[k]:
        precArProt[z].append(peakAreas[j])
        conDataArProt[z].append(condData[j])
        brepsArProt[z].append(breps[j])
        ionArProt[z].append(ionNames[j])
        proNameArProt[z].append(proname[k])
        if k == len(proname)-1:
            precArProt[z].append(peakAreas[k-1])
            conDataArProt[z].append(condData[k-1])
            brepsArProt[z].append(breps[k-1])
            ionArProt[z].append(ionNames[k-1])
            proNameArProt[z].append(proname[k-1])
    elif proname[j] != proname[k]:
        precArProt[z].append(peakAreas[k-1])
        conDataArProt[z].append(condData[k-1])
        brepsArProt[z].append(breps[k-1])
        ionArProt[z].append(ionNames[k-1])
        proNameArProt[z].append(proname[k-1])
        precArProt.append([])
        conDataArProt.append([])
        brepsArProt.append([])
        ionArProt.append([])
        proNameArProt.append([])
        z+=1
    elif k == len(proname)-1:
        precArProt[z].append(peakAreas[k-1])
        conDataArProt[z].append(condData[k-1])
        brepsArProt[z].append(breps[k-1])
        ionArProt[z].append(ionNames[k-1])
        proNameArProt[z].append(proname[k-1])
        z+=1
    else:
        break
    j+=1
    k+=1
brepsNoDups = list(set(brepsAr[0]))
raAr=[]
condAr=[]
numRatio = 1
conditions = 2
x=0
condAr.append('mutant')
condAr.append('WT')

raAr.append('mutant')
raAr.append('WT')
protInfo=[]
indM=[]
indW=[]
i=0
protsar=[]
protRatio=[]
protSDm=[]
protSDw=[]
protSEMm=[]
protSEMw=[]
protMn=[]
protName=[]
while i<len(precArProt):
    assignsM=[]
    assignsW=[]
    logsmM=[]
    logsmW=[]
    arrayM=[]
    arrayW=[]
    m=0
    while m<len(brepsNoDups):
        j=0
        assign1=[]
        assign2=[]
        sm1=[]
        sm2=[]
        while j<len((precArProt[i])):
                
            if all([condAr[0] in conDataArProt[i][j], brepsNoDups[m] == brepsArProt[i][j], ionArProt[i][j] != 'precursor', ionArProt[i][j] != 'precursor [M+1]', ionArProt[i][j] != 'precursor [M+2]']):
                assign1.append(int(precArProt[i][j]))
                #logM=np.log2(float(precArProt[i][j]))
                #sm1.append(logM)
            if all([condAr[1] in conDataArProt[i][j], brepsNoDups[m] == brepsArProt[i][j], ionArProt[i][j] != 'precursor', ionArProt[i][j] != 'precursor [M+1]', ionArProt[i][j] != 'precursor [M+2]']):
                assign2.append(int(precArProt[i][j]))
                #logW=np.log2(float(precArProt[i][j]))
                #sm2.append(logW)
                    
            j+=1
        m+=1

        arrayM.append(assign1)
        arrayW.append(assign2)
        assignsM.append(np.average(assign1))
        assignsW.append(np.average(assign2))
        logsmM.append(sum(assign1))
        logsmW.append(sum(assign2))
        
    protsar.append([proNameArProt[i][0]])

    indM.append(assignsM)
    indW.append(assignsW)

    smpRatio=np.average(logsmM)/np.average(logsmW)
    protRatio.append([smpRatio])

    s=0
    while s<len(brepsNoDups):
        sdArrayM=[]
        sdArrayW=[]
        #fix SD because right now it is an array of 3 different lists
        protMn.append(np.average(arrayM[s])/np.average(arrayW[s]))
        protName.append(proNameArProt[i][0])
        r=0
        while r < len(arrayM[s]):
            sdArrayM.append(np.square(arrayM[s][r]-np.mean(arrayM[s])))
            sdArrayW.append(np.square(arrayW[s][r]-np.mean(arrayW[s])))
            r+=1
        sdfrM = np.sqrt((sum(sdArrayM))/(len(arrayM[s])-1))
        protSDm.append([sdfrM])
        
        sdfrW = np.sqrt((sum(sdArrayW))/(len(arrayW[s])-1))
        protSDw.append([sdfrW])

        SEMm = sdfrM/np.sqrt(len(arrayM[s]))
        SEMw = sdfrW/np.sqrt(len(arrayW[s]))

        protSEMm.append([SEMm])
        protSEMw.append([SEMw])
        s+=1

    i+=1

br=0
ProtLab=[]
ProtLab.append('Protein')
while br<len(brepsNoDups):
    lab0='Mean ' + condAr[0] + brepsNoDups[br]
    ProtLab.append(lab0)
    br+=1
br=0   
while br<len(brepsNoDups):    
    lab1='Mean ' + condAr[1] + brepsNoDups[br]
    ProtLab.append(lab1)
    br+=1
    
ratiolab='Fold Change ' + condAr[0] + ', ' + condAr[1]
sdlab0= 'Standard Deviation ' + condAr[0]
sdlab1= 'Standard Deviation ' + condAr[1]
semlab0= 'SEM ' + condAr[0]
semlab1= 'SEM ' + condAr[1]
ProtLab.append(ratiolab)
ProtLab.append(sdlab0)
ProtLab.append(sdlab1)
ProtLab.append(semlab0)
ProtLab.append(semlab1)

appendInfo2 = sys.argv[10] #raw_input('What would you like to append to the Skyline report? ')
fn= sys.argv[3] + '\\' + sys.argv[2] + '_' + appendInfo2 + 'ProteinLevel.csv'
#fn = tempRfolder + "tempR" + "\\" + "stats_ProteinLevel.csv"
with open(fn, 'wb') as myfile:
    outputFile = csv.writer(myfile)
    i=0
    joinList=[]
    outputFile.writerow(ProtLab)
    while i<len(protsar):
        
        joinList = protsar[i] + indM[i] + indW[i] + protRatio[i] + protSDm[i] + protSDw[i] + protSEMm[i] + protSEMw[i]

        outputFile.writerows([joinList])
        i+=1
    myfile.close()

fn = sys.argv[3] + '\\' + sys.argv[2] + '_statProteinLevel_' + appendInfo + '.csv'
#fn = tempRfolder + "tempR" + "\\" + "stats_ProteinLevel.csv"
with open(fn, 'wb') as myfile:
    outputFile = csv.writer(myfile)
    i=0
    joinList=[]
    outputFile.writerow(ProtLab)
    while i<len(protsar):
        
        joinList = protsar[i] + indM[i] + indW[i] + protRatio[i] + protSDm[i] + protSDw[i] + protSEMm[i] + protSEMw[i]

        outputFile.writerows([joinList])
        i+=1
    myfile.close()
fn= tempRfolder + "tempR" + "\\" + "stats_ProtRscript.csv"
with open(fn, 'wb') as myfile:
    outputFile = csv.writer(myfile)
    i=0
    joinList=[]
    outputFile.writerow(["Protein", "Ratio"])
    while i<len(protName):

        outputFile.writerow([protName[i], protMn[i]])
        i+=1
    myfile.close()
########end Protein section


try:
    #moving R imaging script from Tool Folder to temp directory
    tempFolder = sys.argv[0].strip("DIAprobe.py")
    Rscriptold =  tempFolder + "rimaging.R"
    Rscriptnew = tempRfolder + "tempR"+ "\\" + "rimaging.R"
    rcwd = tempRfolder + "tempR"
    shutil.move(Rscriptold, Rscriptnew)
    Rpath='C:\\Program Files\\R\\R-3.3.0\\bin\\i386\\Rscript.exe'

    s = subprocess.Popen([Rpath, Rscriptnew], cwd=rcwd, shell=False)
    print 'Your graphs are being generated.'

    s.communicate()
    oldImageLocHM=tempRfolder + "tempR"+ "\\" + "heatmap.tiff"
    newImageLocHM= sys.argv[3] + "\\" + sys.argv[2] + '_' + appendInfo2 +"_PeakAreaHeatmap.tiff"
    oldImageLocBox=tempRfolder + "tempR"+ "\\" + "box.pdf"
    newImageLocBox= sys.argv[3] + "\\" + sys.argv[2] + '_' + appendInfo2 +"_ProteinBoxplot.pdf"
    oldImageLocScat=tempRfolder + "tempR"+ "\\" + "scatterplot.png"
    newImageLocScat= sys.argv[3] + "\\" + sys.argv[2] + '_' + appendInfo2 +"_RatioScatterplot.png"
    if os.path.isfile(oldImageLocBox):
        s.terminate()
        shutil.move(oldImageLocBox, newImageLocBox)
        shutil.move(oldImageLocScat, newImageLocScat)
        shutil.move(oldImageLocHM, newImageLocHM)
        print 'Your Statistics File and Graphs have been successfully generated and written to' + fnThree
        
except:
    print 'Make sure your R platform is setup correctly. Please refer to DIAprobe user manual or email adsouza@buckinstitute.org.'
    
##r = R(RCMD="C:\\Program Files\\R\\R-3.2.3\\bin\\R")
##r('library(ggplot2)')
##r('data=read.csv("stats_Rscript.csv")')
##r('test=ggplot(data=data, aes(x=ratio, y=pvalue))+geom_point()')
##r("ggsave(filename = 'test.png')")
